"use strict";
var networkInterface_1 = require('./networkInterface');
var store_1 = require('./store');
var QueryManager_1 = require('./QueryManager');
var lodash_1 = require('lodash');
var ApolloClient = (function () {
    function ApolloClient(_a) {
        var _b = _a === void 0 ? {} : _a, networkInterface = _b.networkInterface, store = _b.store, reduxRootKey = _b.reduxRootKey;
        this.reduxRootKey = reduxRootKey ? reduxRootKey : 'apollo';
        this.networkInterface = networkInterface ? networkInterface :
            networkInterface_1.createNetworkInterface('/graphql');
        if (store &&
            lodash_1.isUndefined(store.getState()[this.reduxRootKey])) {
            throw new Error("Existing store does not use apolloReducer for " + this.reduxRootKey);
        }
        this.store = store ?
            store :
            store_1.createApolloStore(this.reduxRootKey);
        this.queryManager = new QueryManager_1.QueryManager({
            networkInterface: this.networkInterface,
            store: this.store,
            reduxRootKey: this.reduxRootKey,
        });
    }
    ApolloClient.prototype.watchQuery = function (options) {
        return this.queryManager.watchQuery(options);
    };
    ApolloClient.prototype.query = function (options) {
        var _this = this;
        if (options.returnPartialData) {
            throw new Error('returnPartialData option only supported on watchQuery.');
        }
        return new Promise(function (resolve, reject) {
            var handle = _this.queryManager.watchQuery(options);
            handle.onResult(function (result) {
                resolve(result);
                handle.stop();
            });
        });
    };
    ApolloClient.prototype.mutate = function (options) {
        return this.queryManager.mutate(options);
    };
    return ApolloClient;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = ApolloClient;
//# sourceMappingURL=index.js.map